@@ -28,27 +28,55 @@ logger = logging.getLogger(__name__)
 
 class PayPalFormMixin(object):
     """Common methods for the PayPal forms."""
-    def call_paypal(self, post_data):
+    def call_paypal(self, api_url, post_data, transaction=None):
         """
         Gets the PayPal API URL from the settings and posts ``post_data``.
 
+        :param api_url: The API endpoint that should be called.
         :param post_data: The full post data for PayPal containing all needed
           information for the current transaction step.
+        :param transaction: If you already have a transaction, pass it into
+          this method so that it can be logged in case of an error.
 
         """
+        data = urllib.urlencode(post_data)
         try:
-            response = urllib2.urlopen(
-                API_URL, data=urllib.urlencode(post_data))
+            response = urllib2.urlopen(api_url, data=data)
         except (
                 urllib2.HTTPError,
                 urllib2.URLError,
                 httplib.HTTPException), ex:
-            self.log_error(ex)
+            self.log_error(
+                ex, api_url=api_url, request_data=data,
+                transaction=transaction)
         else:
             parsed_response = urlparse.parse_qs(response.read())
             return parsed_response
 
-    def log_error(self, error_message, transaction=None):
+    def get_cancel_url(self):
+        """Returns the paypal cancel url."""
+        return settings.HOSTNAME + reverse(
+            'paypal_canceled', kwargs=self.get_url_kwargs())
+
+    def get_error_url(self):
+        """Returns the url of the payment error page."""
+        return reverse('paypal_error')
+
+    def get_notify_url(self):
+        """Returns the notification (ipn) url."""
+        return settings.HOSTNAME + reverse('ipn_listener')
+
+    def get_return_url(self):
+        """Returns the paypal return url."""
+        return settings.HOSTNAME + reverse(
+            'paypal_confirm', kwargs=self.get_url_kwargs())
+
+    def get_success_url(self):
+        """Returns the url of the payment success page."""
+        return reverse('paypal_success')
+
+    def log_error(self, error_message, api_url=None, request_data=None,
+                  transaction=None):
         """
         Saves error information as a ``PaymentTransactionError`` object.
 
@@ -59,6 +87,8 @@ class PayPalFormMixin(object):
         payment_error = PaymentTransactionError()
         payment_error.user = self.user
         payment_error.response = error_message
+        payment_error.paypal_api_url = api_url
+        payment_error.request_data = request_data
         payment_error.transaction = transaction
         payment_error.save()
         return payment_error
@@ -75,6 +105,7 @@ class DoExpressCheckoutForm(PayPalFormMixin, forms.Form):
     payerID = forms.CharField()
 
     def __init__(self, user, *args, **kwargs):
+        self.user = user
         super(DoExpressCheckoutForm, self).__init__(*args, **kwargs)
         try:
             self.transaction = PaymentTransaction.objects.get(
@@ -90,27 +121,32 @@ class DoExpressCheckoutForm(PayPalFormMixin, forms.Form):
             'TOKEN': self.transaction.transaction_id,
             'PAYERID': self.data['payerID'],
             'PAYMENTREQUEST_0_AMT': self.transaction.value,
-            'PAYMENTREQUEST_0_NOTIFYURL': settings.HOSTNAME + reverse(
-                'ipn_listener'),
+            'PAYMENTREQUEST_0_NOTIFYURL': self.get_notify_url(),
         })
         return post_data
 
     def do_checkout(self):
         """Calls PayPal to make the 'DoExpressCheckoutPayment' procedure."""
         post_data = self.get_post_data()
-        parsed_response = self.call_paypal(post_data)
+        api_url = API_URL
+        parsed_response = self.call_paypal(api_url, post_data)
         if parsed_response.get('ACK')[0] == 'Success':
             transaction_id = parsed_response.get(
                 'PAYMENTINFO_0_TRANSACTIONID')[0]
             self.transaction.transaction_id = transaction_id
             self.transaction.status = PAYMENT_STATUS['pending']
             self.transaction.save()
-            return redirect(reverse('paypal_success'))
+            return redirect(self.get_success_url())
         elif parsed_response.get('ACK')[0] == 'Failure':
-            self.transaction.status = PAYMENT_STATUS['error']
+            self.transaction.status = PAYMENT_STATUS['canceled']
             self.transaction.save()
-            self.log_error(parsed_response, self.transaction)
-            return redirect(reverse('paypal_error'))
+            # we have to do urlencode here to make the post data more readable
+            # in the error log
+            post_data_encoded = urllib.urlencode(post_data)
+            self.log_error(
+                parsed_response, api_url, request_data=post_data_encoded,
+                transaction=self.transaction)
+            return redirect(self.get_error_url())
 
 
 class SetExpressCheckoutFormMixin(PayPalFormMixin, forms.Form):
@@ -166,26 +202,30 @@ class SetExpressCheckoutFormMixin(PayPalFormMixin, forms.Form):
         """Creates the post data dictionary to send to PayPal."""
         post_data = PAYPAL_DEFAULTS
         total_value = 0
+        item_index = 0
         for item, quantity in item_quantity_list:
             if not quantity:
                 # If a user chose quantity 0, we don't include it
                 continue
             total_value += item.value * quantity
             post_data.update({
-                'L_PAYMENTREQUEST_0_NAME0': item.name,
-                'L_PAYMENTREQUEST_0_DESC0': item.description,
-                'L_PAYMENTREQUEST_0_AMT0': item.value,
-                'L_PAYMENTREQUEST_0_QTY0': quantity,
+                'L_PAYMENTREQUEST_0_NAME{0}'.format(
+                    item_index): item.name,
+                'L_PAYMENTREQUEST_0_DESC{0}'.format(
+                    item_index): item.description,
+                'L_PAYMENTREQUEST_0_AMT{0}'.format(
+                    item_index): item.value,
+                'L_PAYMENTREQUEST_0_QTY{0}'.format(
+                    item_index): quantity,
             })
+            item_index += 1
 
         post_data.update({
             'METHOD': 'SetExpressCheckout',
             'PAYMENTREQUEST_0_AMT': total_value,
             'PAYMENTREQUEST_0_ITEMAMT': total_value,
-            'RETURNURL': settings.HOSTNAME + reverse(
-                'paypal_confirm', kwargs=self.get_url_kwargs()),
-            'CANCELURL': settings.HOSTNAME + reverse(
-                'paypal_canceled', kwargs=self.get_url_kwargs()),
+            'RETURNURL': self.get_return_url(),
+            'CANCELURL': self.get_cancel_url(),
         })
         return post_data
 
@@ -193,6 +233,20 @@ class SetExpressCheckoutFormMixin(PayPalFormMixin, forms.Form):
         """Provide additional url kwargs, by overriding this method."""
         return {}
 
+    def post_transaction_save(self, transaction, item_quantity_list):
+        """
+        Override this method if you need to create further objects.
+
+        Once we got a successful response from PayPal we can create a
+        Transaction with status "checkout". You might want to create or
+        manipulate further objects in your app at this point.
+
+        For example you might ask for user's the t-shirt size on your checkout
+        form. This a good place to save the user's choice on the UserProfile.
+
+        """
+        return
+
     def set_checkout(self):
         """
         Calls PayPal to make the 'SetExpressCheckout' procedure.
@@ -202,9 +256,10 @@ class SetExpressCheckoutFormMixin(PayPalFormMixin, forms.Form):
         """
         item_quantity_list = self.get_items_and_quantities()
         post_data = self.get_post_data(item_quantity_list)
+        api_url = API_URL
 
         # making the post to paypal and handling the results
-        parsed_response = self.call_paypal(post_data)
+        parsed_response = self.call_paypal(api_url, post_data)
         if parsed_response.get('ACK')[0] == 'Success':
             token = parsed_response.get('TOKEN')[0]
             transaction = PaymentTransaction(
@@ -216,16 +271,25 @@ class SetExpressCheckoutFormMixin(PayPalFormMixin, forms.Form):
                 content_object=self.get_content_object(),
             )
             transaction.save()
+            self.post_transaction_save(transaction, item_quantity_list)
             for item, quantity in item_quantity_list:
                 if not quantity:
                     continue
-                PurchasedItem.objects.create(
-                    user=self.user, transaction=transaction, item=item,
-                    quantity=quantity)
+                item_kwargs = {
+                    'user': self.user,
+                    'transaction': transaction,
+                    'quantity': quantity,
+                }
+                if item.pk:
+                    item_kwargs.update({'item': item, })
+                PurchasedItem.objects.create(**item_kwargs)
             return redirect(LOGIN_URL + token)
         elif parsed_response.get('ACK')[0] == 'Failure':
-            self.log_error(parsed_response)
-            return redirect(reverse('paypal_error'))
+            post_data_encoded = urllib.urlencode(post_data)
+            self.log_error(
+                parsed_response, api_url=api_url,
+                request_data=post_data_encoded)
+            return redirect(self.get_error_url())
 
 
 class SetExpressCheckoutItemForm(SetExpressCheckoutFormMixin):
